; XINT_REV.LSP
;
; The authors of the XINT.LSP program published in the AutoLISP Bonus section 
; of the February 1993 issue of CADalyst have revised this program so that it 
; calculates the intersections of entities that are on any plane in space as 
; long as both of the entities are on the same plane. The original version of 
; XINT.LSP only worked correctly for entities created in the world coordinate 
; system.  
;
;==============  Intersection of Lines/Arcs/Circles Snap Program  ==============
; Filename: XINT.LSP
; Function: XINT (NF / A1 A2 AL C1 C2 D DC DP DR E1 E2 EL EL1 EL2 ET ET1 ET2
;                  P P1 P2 PP PS1 R1 R2 S)
; Purpose: Selects two entities (lines, arcs, or circles) and returns a point of
;          intersection.  If NF=0 the near point is returned.  If NF=1 the far
;          point is returned.  The point of intersection is found even if the
;          intersection is off the screen.  If the entities will intersect only
;          if they are extended, the theoretical extended intersection will be
;          returned.
; Version: 1.1
; Author: Glenn S. Lyford
; Date: 5/20/90
; Revised: 5/29/90 by Glenn S. Lyford
; Revised: 10/20/90 by George E. Zinsmeister to add documentation and
;   standardize variable names.
; Revised: 12/10/92 by George E. Zinsmeister to allow operation in any UCS and
;   add additional documentation.

;------------------------------------------------------------------------------

; Parameters
; Name               Definition
;------------------------------------------------------------------------------
; NF         : flag to indicate whether near or far intersection is desired
;              N=0 near point is returned; N=1 far point is returned
; -----------------------------------------------------------------------------
; Local Variables
; Variable name      Definition
; -----------------------------------------------------------------------------
; A1, A2, AL : angles, see documentation figures
; C1, C2     : center points of selected entities
; D          : distance from center to closest point of a line
; DC         : distance from first end point of a line to center
; DP         : distance to desired point from PP
; DR         : sum of radii R1 and R2
; E1, E2     : list returned by entsel for first and second entities selected
; EL         : utility variable for swapping variables
; EL1, EL2   : association lists for first and second entities selected
; ENT1, ENT2 : entity names of first and second entities selected
; ET         : utility variable for swapping variables
; ET1, ET2   : type (i.e. line, arc, circle) of first and second entities
;              selected
; P          : returned intersection point; nil is returned if there is no
;              intersection point
; P1, P2     : intersection points when there are two intersection points
; PL1, PL2   : end points of a line
; PP         : closest point of a line
; PNEAR,PFAR : near/far intersection points
; PS1        : point where first entity is selected
; R          : utility variable for swapping variables
; R1, R2     : radii of selected entities
; S          : (/ (+ R1 R2 DC) 2), see documentation figures
;-----------------------------------------------------------------------------
;
(defun XINT (NF / A1 A2 AL C1 C2 D DC DP DR E1 E2 EL EL1 EL2 ENT1 ENT2 ET ET1
  ET2 P P1 P2 PL1 PL2 PP PS1 R R1 R2 S)
  (setq  E1 (entsel "intersection of\n"))
  (if E1
    (progn
      (setq E2 (entsel "and\n"))                           ;select first entity
      (if E2
        (progn
          (setq
            PS1 (cadr E1)     ;get UCS point at which first entity was selected
            ENT1 (car E1)
            EL1 (entget ENT1)                ;association list for first entity
            ENT2 (car E2)
            EL2 (entget ENT2)               ;association list for second entity
            ET1 (cdr (assoc 0 EL1))                       ;type of first entity
            ET2 (cdr (assoc 0 EL2))                      ;type of second entity
          )
         ; swap data if arc/circle and line so that line is first entity - this
                                            ; eliminates some code needed later
          (if (and (/= ET1 "LINE") (= ET2 "LINE"))
            (setq ET ET1 EL EL1 ET1 ET2 EL1 EL2 ET2 ET EL2 EL))
          (cond
            ((and (= ET1 "LINE") (= ET2 "LINE")) (LL_INT))
            ((and (= ET1 "LINE") (or (= ET2 "ARC") (= ET2 "CIRCLE"))) (LA_INT))
            ((and (or (= ET1 "ARC") (= ET1 "CIRCLE"))
               (or (= ET2 "ARC") (= ET2 "CIRCLE"))) (AA_INT))
            (t (setq P nil))
          )
        )                                                            ;end progn
        (prompt "No entity selected\n")
      )                                                              ;end E2 if
    )                                                                ;end progn
    (prompt "No entity selected\n")
  )
)                                                                    ;end defun

;------------------------------------------------------------------------------


;======================  Intersection of Arcs/Circles  =======================

(defun AA_INT ()
  (setq
    C1 (trans (cdr (assoc 10 EL1)) ENT1 1)          ;center of first arc/circle
                                                ;transformed to UCS coordinates
    C2 (trans (cdr (assoc 10 EL2)) ENT2 1)         ;center of second arc/circle
                                                ;transformed to UCS coordinates
    R1 (cdr (assoc 40 EL1))                         ;radius of first arc/circle
    R2 (cdr (assoc 40 EL2))                        ;radius of second arc/circle
    DC (distance C1 C2)                               ;distance between centers
    DR (+ R1 R2)                                              ;sum of the radii
    A1 (angle C1 C2)                             ;angle of line between centers
  )

  (if (< R1 R2)                   ;swap so signs will be correct for finding A2
    (setq R R2 R2 R1 R1 R C1 C2 A1 (+ pi A1)))
  (cond
    ((< DC DR)
      (if (and (>= (+ DC R1) R2) (>= (+ DC R2) R1))        ;calculate if one is
        (progn                                           ;not completely inside
          (setq                                                      ;the other
            S (/ (+ R1 R2 DC) 2)                            ;half the perimeter
            D (- (* R2 R2) (* R1 R1) (* DC DC))          ;difference of squares
            A2 (atan (/ (sqrt (* S (- S R1) (- S R2) (- S DC))) D -0.25))
                                                       ;angle of triangle at C1
            P1 (polar C1 (- A1 A2) R1)                      ;first intersection
            P2 (polar C1 (+ A1 A2) R1)                     ;second intersection
          )
          (if (<= (distance P1 PS1) (distance P2 PS1))      ;set PNEAR and PFAR
            (setq PNEAR P1 PFAR P2)
            (setq PFAR P1 PNEAR P2)
          )
          (if (= NF 0) (setq P PNEAR) (setq P PFAR))      ;choose PNEAR or PFAR
        )                                                            ;end progn
        (setq P nil)                         ;one is inside other so return nil
      )                                                                 ;end if
    )
    ((equal DC DR (* 0.0001 (getvar "dimscale")))                ;tangent point
      (setq P (polar C1 A1 R1))
    )
       ; note that "fuzz" to determine tangency is related to "dimscale" - this
                    ; criterion can be change according to specific application
    (setq P nil)                                                 ;too far apart
  )
)                                                                    ;end defun
;------------------------------------------------------------------------------

;==================  Intersection of Line with Arc/Circle  ====================

(defun LA_INT ()
  (setq
    PL1 (trans (cdr (assoc 10 EL1)) 0 1)    ;start point of line transformed to
                                                               ;UCS coordinates
    PL2 (trans (cdr (assoc 11 EL1)) 0 1)      ;end point of line transformed to
                                                               ;UCS coordinates
    C2 (trans (cdr (assoc 10 EL2)) ENT2 1)                ;center of arc/circle
                                                ;transformed to UCS coordinates
    R2 (cdr (assoc 40 EL2))                               ;radius of arc/circle
    AL (angle PL1 PL2)                                  ;absolute angle of line
    A1 (angle PL1 C2)           ;absolute angle from first line point to center
    A2 (abs (- A1 AL))             ;relative angle between line and pt-ctr line
    DC (distance PL1 C2)                   ;distance from first point to center
    D (* DC (sin A2))            ;distance of center from closest point of line
    PP (polar PL1 AL (* DC (cos A2)))                    ;closest point of line
  )
  (cond
    ((< D R2)
      (progn                         ;otherwise calculate point of intersection
        (setq
          DP (sqrt (- (* R2 R2) (* D D)))    ;distance of desired point from PP
          P1 (polar PP AL DP)                          ;point on one side of PP
          P2 (polar PP (+ AL pi) DP)                 ;point on other side of PP
        )
        (if (<= (distance P1 PS1) (distance P2 PS1))        ;set PNEAR and PFAR
          (setq PNEAR P1 PFAR P2)
          (setq PFAR P1 PNEAR P2)
        )
        (if (= NF 0) (setq P PNEAR) (setq P PFAR))        ;choose PNEAR or PFAR
      )                                                              ;end progn
    )
    ((equal D R2 (* 0.0001 (getvar "dimscale"))) (setq P PP))    ;tangent point
    ; note that "fuzz" to determine tangency is related to "dimscale" - this
    ; criterion can be changed according to specific application
    (T (setq P nil))                             ;no intersection so return nil
  )                                                                   ;end cond
)                                                                    ;end defun
;------------------------------------------------------------------------------

;=========================  Intersection of Lines  ==========================

(defun LL_INT ()
  (setq P
    (inters
      (trans (cdr (assoc 10 EL1)) 0 1)
      (trans (cdr (assoc 11 EL1)) 0 1)
      (trans (cdr (assoc 10 EL2)) 0 1)
      (trans (cdr (assoc 11 EL2)) 0 1)
      nil
    )
  )
)                                                                    ;end defun

;------------------------------------------------------------------------------

